﻿// Repräsentiert ein Sudoku-Feld
function SudokuField(r,c){
	this.row = r;
	this.col = c;
	this.value = "";
	this.solution = "";
	this.readonly = true;
}
// Prototyp-Methode zur Anzeige des Wertes im Sudoku
SudokuField.prototype.display = function(){
	var input = $("#"+this.row+"-"+this.col);
	input.val(this.value);
	if (this.readonly){
		input.attr("readonly", "true");
		input.addClass("readonly");
	}
	else {
		input.removeAttr("readonly");
		input.removeClass("readonly");
	}
}
SudokuField.prototype.markHelp = function(){
	var input = $("#"+this.row+"-"+this.col);
	input.addClass("helpmark");
}
SudokuField.prototype.solveHelp = function(){
	var input = $("#"+this.row+"-"+this.col);
	input.removeClass("helpmark");
	if (this.value == "")
		this.value = this.solution;
	else
		this.value = "";
	this.display();
}
// Repräsentiert das Sudoku-Spiel
function Sudoku(){
	var fields = []; // privates Array von Sudoku-Zeilen
	for (var r = 1; r <= 9; r++){
		var row = []; // Sudoku-Zeile mit Sudoku-Feldern
		for (var c = 1; c <= 9; c++)
			row.push(new SudokuField(r,c));
		fields.push(row);
	}
	function getRow(field){ // Liefert andere Felder der Zeile
		return fields[field.row-1].filter(function(f){
			return f != field;
		});
	}
	function getCol(field){ // Liefert anderer Felder der Spalte
		return fields.reduce(function(acc,cur){
			if (cur[field.col-1] != field)
				acc.push(cur[field.col-1]);
			return acc;
		},[]);
	}
	function getBlock(field){ // Liefert andere Felder des Blocks
		var fromRow = Math.floor((field.row-1)/3)*3;
		var fromCol = Math.floor((field.col-1)/3)*3;
		var result = [];
		for (var x = fromRow; x < fromRow+3; x++){
			for (var y = fromCol; y < fromCol+3; y++){
				if (fields[x][y] != field)
					result.push(fields[x][y]);
			}
		}
		return result;
	}
	function check(getFields, field, num){ // prüft ob num bereits vorhanden
		var checkFields = getFields(field); // Ermittlung der Prüf-Felder durch übergebene Funktion
		for (var i = 0; i < checkFields.length; i++){
			if (checkFields[i].value == num)
				return false;
		}
		return true;
	}
	function initField(field, value){ // versucht ein Feld mit einem Wert zu initialisieren
		if (check(getRow, field,value) && 
			check(getCol, field,value) && 
			check(getBlock, field, value)){
			field.value = value; // wenn der Wert in Zeile, Spalte und Block nicht vorkommt
			return true;
		}
		return false;
	}
	function fill(){ // Füllt das Sudoku zufällig, aber nach den Sudoku-Regeln
		// Erzeugung einer flachen Liste aller Sudoku-Felder
		var flatFields = fields.reduce(function(acc,cur){
			return acc.concat(cur);
		},[]);
		for (var i = 0; i < flatFields.length; i++){
			var field = flatFields[i];
			if (field.options == undefined) // Initialisierung der möglichen Werte für das Feld
				field.options = [1,2,3,4,5,6,7,8,9];
			if (field.options.length > 0){ // Es gibt noch Möglichkeiten
				var idx = Math.floor(Math.random()*field.options.length);
				while(!initField(field, field.options[idx])){ // Fehlschlag der Initialisierung?
					field.options.splice(idx,1); // Möglichkeit entfernen
					if (field.options.length == 0)
						break; // keine Möglichkeit mehr
					idx = Math.floor(Math.random()*field.options.length);
				}
			}
			if (field.options.length == 0){ // Keine Möglichkeit der Feldinitialisierung
				// Backtracking
				delete field.options;
				var prevField = flatFields[i-1]; // das vorherige bereits initialisierte Feld
				// Alter Initialisierungswert wird aus den Möglichkeiten entfernt
				prevField.options.splice(prevField.options.indexOf(prevField.value),1);
				prevField.value = ""; // Wert wird zurückgesetzt
				i-=2; // Schleife setzt beim vorherigen Feld fort
			}
		}
		for (var i = 0; i < flatFields.length; i++){
			var field = flatFields[i];
			field.solution = field.value; // Übernahme der korrekten Lösung
			delete field.options; // temporäre Optionen nicht mehr notwendig
		}
	}
	function getOptions(field){ // Liefert die möglichen Werte für das Feld
		function reduceEmptySolutions(acc, cur){
			if (cur.value == "")
				acc.push(cur.solution);
			return acc;
		}
		// Ermittlung der Optionen in Zeile, Spalte und Block
		var rowOptions = getRow(field).reduce(reduceEmptySolutions,[]);
		var colOptions = getCol(field).reduce(reduceEmptySolutions,[]);
		var blockOptions = getBlock(field).reduce(reduceEmptySolutions,[]);
		var result = [field.solution]; // Eigene solution is immer eine Möglichkeit
		for (var b = 0; b < blockOptions.length; b++){
			var opt = blockOptions[b];
			if (rowOptions.indexOf(opt) != -1 && 
				colOptions.indexOf(opt) != -1)
				result.push(opt); // opt ist eine Option in Zeile und Spalte und Block
		}
		return result;
	}
	function isSolved(){ // Prüft ob das Sudoku richtig ausgefüllt ist
		for (var r = 0; r < 9; r++){
			for (var c = 0; c < 9; c++){
				var field = fields[r][c];
				if (field.value != field.solution)
					return false;
			}
		}
		return true;
	}
	function canSolve(){ // Prüft ob das Sudoku eindeutig lösbar ist
		var resetFields = [];
		var canSet = true;
		while(canSet){
			canSet = false;
			for (var r = 0; r < 9; r++){
				for (var c = 0; c < 9; c++){
					var field = fields[r][c];
					if (field.value != "")
						continue;
					var opt = getOptions(field);
					if (opt.length == 1){ // nur eine Lösung möglich
						field.value = opt[0];
						resetFields.push(field);
						canSet = true;
					}
				}
			}
		}
		var res = isSolved(); // gelöst, wenn nichts mehr gesetzt werden kann?
		for (var i = 0; i < resetFields.length; i++)
			resetFields[i].value = ""; // gelöste Felder zurücksetzen
		return res;
	}
	function clear(maxMoves){ // freie Felder erzeugen, solange es lösbar bleibt
		if (maxMoves == undefined) maxMoves = 100;
		var flatFields = fields.reduce(function(acc,cur){
			return acc.concat(cur);
		},[]);
		var checkIndex = flatFields.map(function(cur,idx){return idx;}); // [0,1,...,80]
		while(checkIndex.length > 0)
		{
			var idx = Math.floor(Math.random()*checkIndex.length);
			var field = flatFields[checkIndex[idx]]; // zufälliges Feld
			field.value = ""; // leeren
			if (!canSolve()) // wenn nicht mehr lösbar
				field.value = field.solution; // Wert wiederherstellen
			else{
				field.readonly = false; // ein bearbeitbares Feld
				maxMoves--;
			}
			checkIndex.splice(idx,1); // Feld nicht erneut prüfen
			if (maxMoves == 0)
				break;
		}
	}
	this.create = function(maxMoves){
		fill(); // zufällig nach den Sudoku-Regeln füllen
		clear(maxMoves); // zufällig Werte entfernen
	}
	this.show = function(){
		for (var r = 0; r < 9; r++){
			for (var c = 0; c < 9; c++)
				fields[r][c].display();
		}
	}
	this.set = function(id, value){ // eingebenen Wert setzen
		var r = id[0];
		var c = id[2];
		var field = fields[r-1][c-1];
		field.value = value;
		return isSolved(); // wurde das Sudoku gelöst?
	}
	this.markHelp = function(){ // Hilfe-Markierung setzen
		for (var r = 0; r < 9; r++){
			for (var c = 0; c < 9; c++){
				var field = fields[r][c];
				if (field.value != "" && field.value != field.solution){
					field.markHelp(); // Ein falsches Feld markieren
					return;
				}
			}
		}
		for (var r = 0; r < 9; r++){
			for (var c = 0; c < 9; c++){
				var field = fields[r][c];
				if (field.value == "" && getOptions(field).length == 1){
					field.markHelp(); // Ein eindeutiges Feld markieren
					return;
				}
			}
		}
	}
	this.solveHelp = function(){
		var solveItem = $(".helpmark");
		if (solveItem){
			var id = solveItem[0].id;
			var r = id[0];
			var c = id[2];
			var field = fields[r-1][c-1];
			field.solveHelp();
		}
		return isSolved();
	}
	this.toJSON = function(){
		return JSON.stringify(fields);
	}
	this.fromJSON = function(str) {
		var f = JSON.parse(str);
		for (var r = 0; r < 9; r++){
			for (var c = 0; c < 9; c++){
				var field = fields[r][c];
				field.value = f[r][c].value;
				field.solution = f[r][c].solution;
				field.readonly = f[r][c].readonly;
			}
		}
	}
}
